#ifndef MOTestFunctions_
#define MOTestFunctions_ 

#include "mies/SerialES/MixedIntegerES.h"
extern "C" 
{
#include "nsga2-v1.1/global.h" // Included for PI constant
}

class MOTestFunctions 
{
  public:
    MOTestFunctions() { F.resize(3, 0); }

    void DTLZ1(Individual* I) { DTLZ1(I, NULL, I->R.size(), NULL); }
    void DTLZ1(double* xreal, int nreal) { DTLZ1(NULL, xreal, nreal, NULL); }

    void DTLZ2(Individual* I) { DTLZ2(I, NULL, I->R.size(), NULL); }
    void DTLZ2(double* xreal, int nreal) { DTLZ2(NULL, xreal, nreal, NULL); }

    vector<double>* getF() { return &F; }
  protected:
    vector<double> F;

    /** \brief  (Slightly adjusted) DTLZ1 implementation from PISA (www.tik.ee.ethz.ch/pisa/),
     *          dtlz_c_source by Marco Laumanns, laumanns@tik.ee.ethz.ch
     *
     * Domain: [0,1]^n, n >=5, (see Deb, 2002: Scalable MO Test Problems)
     * reference point (0.7,0.7,0.7) (see Deb, 2005: Evaluating the e-Domination Based MOEA)
     */
    void DTLZ1(Individual* I, double* xreal, int n, double* inputNoise); 

    /** \brief  (Slightly adjusted) DTLZ2 implementation from PISA (www.tik.ee.ethz.ch/pisa/),
     *          dtlz_c_source by Marco Laumanns, laumanns@tik.ee.ethz.ch
     *
     * Domain: [0,1]^n, n >=12, (see Deb, 2002: Scalable MO Test Problems)
     * reference point (1.1,1.1,1.1) (see Naujoks, 2005: Multi-objective Optimisation Using S-Metric)
     */
    void DTLZ2(Individual* I, double* xreal, int n, double* inputNoise); 
};

#endif
