#ifndef RobustLHSR_
#define RobustLHSR_ 

#include "Robust.h"

class RobustLHSR:public Robust 
{
  public: 
    RobustLHSR(unsigned sampleSetSize_, double stdInputNoise_):Robust(sampleSetSize_, stdInputNoise_) { evalsUsed = 0; }
    virtual ~RobustLHSR();

    void robustSimulate(int* xnom, int nnom) { robustSimulate(NULL, xnom, nnom); }

    unsigned getEvalsUsed() { return evalsUsed; }
    void updateTotalGen(unsigned& totalGen, unsigned currGen, unsigned evaluations, unsigned lambda);

  protected: 
    struct Point
    {
      vector<double> R;
      vector<int> Z, D;

      Point(vector<double>* R_, vector<int>* Z_, vector<int>* D_) 
      { 
        if (R_ != NULL)
          R = (*R_);

        if (Z_ != NULL)
          Z = (*Z_);

        if (D_ != NULL) // FIXME !!! work with noise on the Diameters instead of D values (see RobustMEM)...
          D = (*D_);
      }
    };

    struct ArchiveTuple:public Point
    {
      vector<double> F;	    

      ArchiveTuple(Point& point, vector<double>* F_):ReferencePoint(&point.R, &point.Z, &point.D)
      {
        F = (*F_);	
      }
    };

    unsigned evalsUsed;
    vector<ArchiveTuple*> archive;
    
    vector<Point*> referenceSet;
    vector<int> selectedArchiveTuples;
    vector<Point*> candidates;

    void robustSimulate(Individual* I, int* xnom, int numVar); 

    //! Approximate Latin Hypercube reference set by selecting suitable ArchiveTuples
    void archiveSelection(Individual* I, int* xnom, int numVar); 
    
    //! Generate Latin Hypercube reference set based on supplied object variables and stdInputNoise; for discrete nominal object variables
    void LHS(Individual* I, int* xnom, int numVar);  

    //! Retrieve ArchiveTuple closest to Point refSetIndex in the reference set 
    unsigned getClosestArchiveTuple(unsigned refSetIndex); 

    /** \brief  Calculate Euclidian distance between Point refSetIndex in the reference set and ArchiveTuple archiveIndex in the archive; 
     *          for discrete nominal object variables */
    double EuclidianDistanceD(unsigned refSetIndex, unsigned archiveIndex); 
    
    //! Check that ArchiveTuple archiveIndex is not closer to another Point in the reference set 
    bool noCloserRefPoint(refSetIndex, archiveIndex); 
	 
    //! Retrieve probability of I->D (or xnom) being perturbed into archive[selectedArchiveTuples[i]]->D 
    double weightD(vector<int>& D, Individual* I, int* xnom); 

    void cleanupAfterSim();
};

#endif
