#include "RobustMEM.h"

//------------------------------------------------------------------------------
// Class RobustMEM
//-------------------------------------------------
// Public Members

/* Generate sample set using Latin Hypercube sampling or Monte Carlo sampling */
void RobustMEM::generateSampleSet()
{
#ifdef DEBUG
  cout << "RobustMEM::generateSampleSet()" << endl;
#endif

  static int* numNoiseNorm = getNumNoiseNorm(),
            * numNoiseUnif = getNumNoiseUnif();
  
  static int numNoise = ((numNoiseNorm != NULL) ? *numNoiseNorm : 0) + ((numNoiseUnif != NULL) ? *numNoiseUnif : 0);

  if (LH)
    Robust::generateSampleSet_LH(numNoise, numNoiseNorm, numNoiseUnif);
  else
    Robust::generateSampleSet_MC(numNoise, numNoiseNorm, numNoiseUnif);	  
}

//------------------------------------------------------------------------------
// Class RobustMEM
//-------------------------------------------------
// Protected Members

void RobustMEM::robustSimulate(Individual* I, int* xnom, int numVar) 
{
#ifdef DEBUG
  cout << "RobustMEM::robustSimulate()" << endl;
#endif
  
  static vector<double>* F = getF();
  if (numF == 0)
    numF = F->size();
  vector<double> F_eff(numF, 0);

  for (unsigned i = 0; i < sampleSetSize; i++)
  {
    /* Simulate with noise from sample i */
    simulate(I, xnom, numVar, sampleSet[i]);

    /* Sum up resulting fitness values of the different samples */
    for (int j = 0; j < numF; j++)
      F_eff[j] += F->at(j); 
  }

  /* Determine effective fitness by dividing by sampleSetSize */
  for (int j = 0; j < numF; j++)
    F->at(j) = F_eff[j] /= sampleSetSize;

  if (I != NULL)
    I->F = F_eff;
  
#ifdef ROBUSTNESS_EVAL 
  /* In case of robustness evaluation, use distinct sampleSet per simulated individual */
  generateSampleSet(); 
#endif
}

