#!/bin/csh -f

set numRuns = 30
set sleepInterval = 1
set snapShotsFile = "generate_snap_shots"
set executable = "./MOEA"
set distributable = "MOEA_distributable_*"
set processScript = "./processRuns.pl"
set runFiles = "RunID*"
set testrunsDir = "testruns"
set testrunsDirInteger = "testruns_integer"
set testrunsFolderFile = "testruns_folder_structure.tar.gz"
set numNotMIES = 2

set MOEAs = ("NSGAII" "SMSEMOA" "NSGAII_sa_sp_lambda" "SMSEMOA_sa") #"SMSEMOA_dp"[insert directly after SMSEMOA, numNotMies=3] "NSGAII_sa" "NSGAII_sa_sp" "SMSEMOA_sa_dp") 
set compileOpt = ("nsga=1" "sms3d=1" "nsga=1 mies=1 sp=1 lambda=1" "sms3d=1 mies=1") # "sms3d=1 dp=1"[insert directly after SMSEMOA, numNotMies=3] "nsga=1 mies=1" "nsga=1 mies=1 sp=1" "sms3d=1 mies=1 dp=1")
set WDNproblems = ("TwoLoop" "Hanoi" "NewYorkCity")

echo '\n'Preparing $testrunsDir folder structure...'\n'
tar xzf $testrunsFolderFile -C ./

echo '\n'Preparing $testrunsDirInteger folder structure...'\n'
mkdir $testrunsDirInteger
tar xzf $testrunsFolderFile -C $testrunsDirInteger
mv $testrunsDirInteger/$testrunsDir/* $testrunsDirInteger
rm -r $testrunsDirInteger/$testrunsDir

set i = 1
set j = 0
foreach MOEA ($MOEAs)
  echo '\n'Building $MOEA...'\n'
  make clean > /dev/null
  sleep $sleepInterval
  make $compileOpt[$i] > /dev/null

  foreach WDNproblem ($WDNproblems)
    echo '\n'Executing $MOEA - $WDNproblem nominal snapshot run...'\n'
    echo > $snapShotsFile
    $executable $WDNproblem.inp
    rm $snapShotsFile

    @ j = 1
    while ($j <= $numRuns)
      echo '\n'Executing $MOEA - $WDNproblem nominal run $j of $numRuns...'\n' 
      $executable $WDNproblem.inp
      sleep $sleepInterval
      @ j =  $j + 1
    end
    mv $runFiles $testrunsDir/$WDNproblem\_$MOEA

    if ($i > $numNotMIES) then
      echo '\n'Executing $MOEA - $WDNproblem integer snapshot run...'\n'
      echo > $snapShotsFile
      $executable $WDNproblem\_integer.inp
      rm $snapShotsFile

      @ j = 1
      while ($j <= $numRuns)
        echo '\n'Executing $MOEA - $WDNproblem integer run $j of $numRuns...'\n' 
        $executable $WDNproblem\_integer.inp
        sleep $sleepInterval
        @ j =  $j + 1
      end
      mv $runFiles $testrunsDirInteger/$WDNproblem\_$MOEA
    endif
  end

  @ i = $i + 1
end

echo '\n'

# Processing of nominal test runs
$processScript N    # Hypervolume calculation, box plots generation, stepsize plots generation
$processScript N cd # Convergence data calculation
$processScript N cp # Convergence plots generation
$processScript N ar # Attainment surfaces boundary points determination
$processScript N ag # Attainment surfaces generation

# Processing of integer test runs
$processScript I    # Hypervolume calculation, box plots generation, stepsize plots generation
$processScript I cd # Convergence data calculation
$processScript I cp # Convergence plots generation
$processScript I ar # Attainment surfaces boundary points determination
$processScript I ag # Attainment surfaces generation

rm $distributable
make dist > /dev/null
tar --ignore-failed-read -czf $testrunsDir\_`date +'20%y-%m-%d_%H%Mh'`.tar.gz $testrunsDir $distributable
tar --ignore-failed-read -czf $testrunsDirInteger\_`date +'20%y-%m-%d_%H%Mh'`.tar.gz $testrunsDirInteger $distributable
rm $distributable
