#include "NSGAII_MIES.h"

//------------------------------------------------------------------------------
// Class NSGAII_MIES
//-------------------------------------------------
// Protected Members

/* Determine bestF value(s) for current population; only necessary for initial generation since otherwise taken care of in select() */
void NSGAII_MIES::determineBestF()
{
#ifdef DEBUG
  cout << "NSGAII_MIES::determineBestF()" << endl;
#endif

  /* Compare Individuals */
  for (unsigned i = 0; i < mu; i++)
  {
    for (unsigned j = 0; j < mu; j++)
    {
      if (i == j)
        continue;

      compare(P[i], P[j], bestF, false, false);
    }
  }
}

/* In case of convergence study (i.e., with known optima supplied) determine whether optima have been reached */
bool NSGAII_MIES::optimumReached()
{
#ifdef DEBUG
  cout << "NSGAII_MIES::optimumReached()" << endl;
#endif

  vector<bool> reached(selectDimension, false);

  if (generations == USHRT_MAX) // Convergence study
  {
    for (unsigned i = 0; i < selectDimension; i++)
    {
      if (direction[selectFunction[i]] == 1)
      {
        if (bestF[selectFunction[i]] >= optimalF[selectFunction[i]])
	  reached[i] = true;
      }
      else
      {
        if (bestF[selectFunction[i]] <= optimalF[selectFunction[i]])
	  reached[i] = true;
      }
    }
  }
  else                          // Study with fixed number of evaluations
    return false;

  for (unsigned i = 0; i < selectDimension; i++)
  {
    if (!reached[i])
      return false;
  }

  return true;
}

