#ifndef SMS_STEADY2D_MIES
#define SMS_STEADY2D_MIES

#include "NSGAII_Steady_MIES.h"
#include "SMSEMOA_2D_MIES.h"

/** \brief  Steady state (mu + (1, lambda))-MIES version of 2D SMS-EMOA, abstract class
 *          (see NSGAII_MIES.h) */
class SMSEMOA_Steady2D_MIES:public NSGAII_Steady_MIES, public SMSEMOA_2D_MIES 
{                                                                              
  public:                                                                      
    SMSEMOA_Steady2D_MIES(unsigned n_r_, unsigned n_z_, unsigned n_d_, unsigned n_sigma_r_, unsigned n_sigma_z_, unsigned n_prob_, unsigned n_f_, 
		           unsigned mu_, unsigned rho_, unsigned kappa_, unsigned lambda_,  
		             vector<double>& lBound_r_, vector<double>& uBound_r_, vector<int>& lBound_z_, vector<int>& uBound_z_, 
		               vector<int>& lBound_d_, vector<int>& uBound_d_, double prob_min_, double prob_max_,
		                 bool disRec_r_, bool disRec_z_, bool disRec_sigma_r_, bool disRec_sigma_z_, bool disRec_prob_,
		                   bool selfAdaptation_, vector<int>& direction_, 
		                     unsigned evaluations_, vector<double>& optimalF_, 
			               double initialSigma_r_, double initialSigma_z_, double initialProb_,
			                 ifstream* initialPopFile_, int randomSeed_, bool feedback_, 
			                   unsigned timeOutValue_,
		    	                     unsigned selectDimension_, vector<unsigned>& selectFunction_,
					       bool excludeParent_) 
      :NSGAII_Steady_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		           mu_, rho_, kappa_, lambda_,  
		             lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		               lBound_d_, uBound_d_, prob_min_, prob_max_,
			         disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			           selfAdaptation_, direction_, 
			             evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
			               initialPopFile_, randomSeed_, feedback_, timeOutValue_,
			                 selectDimension_, selectFunction_,
					   excludeParent_),
       SMSEMOA_2D_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		        mu_, rho_, kappa_, lambda_,  
		          lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		            lBound_d_, uBound_d_, prob_min_, prob_max_,
			      disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
			        selfAdaptation_, direction_, 
			          evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
			            initialPopFile_, randomSeed_, feedback_, timeOutValue_,
			              selectDimension_, selectFunction_),
       NSGAII_MIES(n_r_, n_z_, n_d_, n_sigma_r_, n_sigma_z_, n_prob_, n_f_, 
		    mu_, rho_, kappa_, lambda_,  
		      lBound_r_, uBound_r_, lBound_z_, uBound_z_, 
		        lBound_d_, uBound_d_, prob_min_, prob_max_,
		          disRec_r_, disRec_z_, disRec_sigma_r_, disRec_sigma_z_, disRec_prob_,
		            selfAdaptation_, direction_, 
		              evaluations_, optimalF_, initialSigma_r_, initialSigma_z_, initialProb_,
		                initialPopFile_, randomSeed_, feedback_, timeOutValue_,
		                  selectDimension_, selectFunction_) {}

    ~SMSEMOA_Steady2D_MIES() {}

  protected:
    /** \brief  Prepare for calculating comparison scores (if necessary) */
    void prepComparisonScore(vector<Individual*>& Q); 

    /** \brief  Calculate 2D HV contr. approximations for pre-selecting "best" offspring Individual */
    double calcComparisonScore(unsigned offspringIndex);
};

#endif
