#include "global.h"

base_generator_type randGenerator(42u);
boost::uniform_real<> uniDistReal(0, 1);
boost::variate_generator<base_generator_type&, boost::uniform_real<> > uniReal(randGenerator, uniDistReal);
boost::uniform_int<> uniDistInt(0, INT_MAX);
boost::variate_generator<base_generator_type&, boost::uniform_int<> > uniInt(randGenerator, uniDistInt);
boost::normal_distribution<> normDist(0, 1);
boost::variate_generator<base_generator_type&, boost::normal_distribution<> > normReal(randGenerator, normDist);
boost::math::normal_distribution<> normDistMath(0, 1);

/* Generate random permutation of 1,...,n in vector A */
void randPerm(vector<int>& A, int n) 
{
  for (int i = 0; i < n; i++)
    A[i] = i+1;

  int r, temp;
  for (int i = n-1; i >= 0; i--) 
  {
    r = uniInt() % (i + 1); // 0 <= r <= i

    temp = A[i];
    A[i] = A[r];
    A[r] = temp;
   }
}

string itos(int number)
{
  ostringstream buffer;
  
  buffer << number;

  return buffer.str();
}

void skipLinesUntil(ifstream& infile, string text)
{
  char line[LEN];
  string temp;
  
  do
  {
    infile.getline(line, LEN);
    temp = line;
  } while (temp.find(text) == string::npos); 
}

char* getValue(ifstream& infile)
{
  char line[LEN], * first;

  /* Skip line if starts with ; */
  do
  {
    infile.getline(line, LEN);
  } while (line[0] == COMMENT); 

  first = strtok(line, DELIMS);

  /* If REPEAT character is found in first token return it */
  if (line[0] == REPEAT)
    return first;

  /* Retrieve value = second token */
  return strtok(NULL, DELIMS);
}

string getStringValue(ifstream& infile)
{
  char line[LEN];

  /* Skip line if starts with ; */
  do
  {
    infile.getline(line, LEN);
  } while (line[0] == COMMENT); 

  /* Retrieve value = second token */
  strtok(line, DELIMS);
  string value = strtok(NULL, DELIMS);
  return value;
}

void getBound(ifstream& infile, vector<double>* bound_r, vector<int>* bound_d_z, int n)
{
  char* value;
  bool repeat = false;

  if (bound_r != NULL)
  {
    for (int i = 0; i < n; i++)
    {
      if (!repeat)
      {
        value = getValue(infile);

        if (value[0] != REPEAT)
          bound_r->push_back(strtod(value, NULL));
        else
	{
          repeat = true;
	  i--;
	}
      }
      else
        bound_r->push_back(bound_r->at(i-1));
    }
  }
  
  if (bound_d_z != NULL)
  {
    for (int i = 0; i < n; i++)
    {
      if (!repeat)
      {
        value = getValue(infile);

        if (value[0] != REPEAT)
          bound_d_z->push_back(atoi(value));
        else
	{
          repeat = true;
	  i--;
	}
      }
      else
        bound_d_z->push_back(bound_d_z->at(i-1));
    }
  }
}

void findAndReplace(string& text, const char* find, const char* replace)
{ 
  if (strlen(find) != strlen(replace))
  {
    cerr << "findAndReplace(): find and replace must be of equal length!" << endl;
    return;
  }

  unsigned pos = text.find(find);

  while (pos != string::npos)
  {
    text.replace(pos, strlen(find), replace);
    pos = text.find(find);
  }
}
