#!/bin/csh -f

if ($# < 3) then
  echo "Usage: $0 N|I WDNproblem *bpData*.txt"
  exit
endif

set plotDir     = "plot"
set ghostScript = "gs"
set gnuplot     = "gnuplot"

$gnuplot << EOF
set term postscript landscape color enhanced dashed linewidth 1 "Arial" 40 

set boxwidth 0.7 relative
set xrange [0:3] 
set yrange [*:*] 

set xtics nomirror
set ytics auto
set format y "%4.2f"
set mytics 0

set xtics out
set xtics scale 0.5 

set grid xtics ytics lt 0 lw 1 lc rgb "#C3C3C3

set ylabel "{/Arial-Bold hypervolume}"

type = "$1"

if (type eq "N") set xtics ("NSGA-II" 1, "SMS-\nEMOA" 2); else set xtics ("NSGA-II\nsa sp\n{/Symbol-Oblique=26 l}" 1, "SMS-\nEMOA\nsa" 2);

set xtics font 'Arial-Italic-Bold, 42'

set size 1.08,1.12
set origin 0.005,-0.06

set lmargin 6.5
set rmargin 2

set output "$plotDir/boxplot.ps"

plot '$3' using 1:3:2:6:5 with candlesticks lt 1 lw 8 lc rgb "#778899" notitle whiskerbars ,      ''                 using 1:4:4:4:4 with candlesticks lt -1 lw 6 notitle
EOF

"$ghostScript" -dSAFER -dBATCH -dNOPAUSE -sDEVICE=pdfwrite -sOutputFile=$plotDir/boxplot$2.pdf $plotDir/boxplot.ps

rm $plotDir/boxplot.ps

